# Wake-informed Lifting Line Theory: Example Code

The wake-informed lifting line method is a tool that can be used to obtain blade loading distributions from slipstream velocity measurements. This MATLAB code serves as an example implementation of the method and is not intended for direct use with external datasets.

Users must adapt the code to use it for their dataset.

## Requirements

This code has been developed in MATLAB R2021b, and tested in both MATLAB R2021b and R2023b. 


## Installation

No additional installation is required for the code except an activated installation of MATLAB (preferably 2021b or higher) with the **Statistics and Machine Learning Toolbox**, **Curve Fitting Toolbox** and **Optimization Toolbox**.


## Structure

```
.
├── CITATION.cff
├── data
│   ├── PropellerGeometry
│   │   ├── xprop_bladeContour.csv
│   │   ├── xprop_faring1.csv
│   │   ├── xprop_faring2.csv
│   │   ├── XProp_geometry.csv
│   │   └── xprop_spinNac_fullLength.csv
│   └── SRANS
│       ├── phaseLocked_J0.60V30b15.xlsx
│       ├── phaseLocked_J1.10V30b15.xlsx
│       ├── ThrustTorque_perBlade_J0.60V30b15.csv
│       └── ThrustTorque_perBlade_J1.10V30b15.csv
├── LICENSES  
│   ├── Apache-License-v2.0.txt
│   └── CC-BY-4.0.txt
├── outputs
│   ├── matFiles
│   └── plots
├── README.md
└── src
    ├── biotSavart.m   
    ├── biotSavartBound.m
    ├── curveFittingPlot.m
    ├── error_velocity.m
    ├── findBladeWake_stats.m
    ├── helixWakePlot.m
    ├── InductionMatrix.m
    ├── LILSolution.m
    ├── MainFile.m
    ├── matlabPlotSettings.m
    ├── plotGeom_2D.m
    ├── propellerGeoemtry.m
    ├── rotate.m
    ├── SetupControlPoints.m
	├── SetupHelicalSystem_J60_noContraction.m
	├── SetupHelicalSystem_J110_noContraction.m
	├── simps.m
	├── SolveWakeInformedLL.m
	├── vortexLine.m
	└── vortexLineLambOseen.m

```

Propeller geomtery details are optional for the code. If known then it can be used to supplement the code.
The `./data/PropellerGeometry/XProp_geometry.csv` file provides the example propeller geometry. This file contains the following variables in the specified order:
- `Station` is the name of different sections of the blades.
- `R/Rp` is the non-dimensionalized radial location of the station, normalized with propeller radius. 
- `R[mm]` is the dimensionalized radial location of the station in millimeters.
- `Twist[deg]` is the twist of the blade sections in degrees with respect to the reference location (0.7R for the example geometry).
- `Chord[mm]` is the local chord size of the blade sections in millimeters.

The other files in the `./data/PropellerGeometry` directory provide the outline of the propeller blade and nacelle for plotting purposes only.

The `./data/SRANS/` directory contains the example SRANS slipstream data, which are used to calculate the loading distributions along the blade span. Additionally, the blade loading obtained from the SRANS simulations is also present in the folder, which acts as validation data as well as input to the load-informed lifitng solution for **LILSolution.m** file.

The **Mainfile.m** in `./src/` is the script to run. With the default settings in the code, the results will be saved in a mat file in the specified output folder (by default, in the `./outputs/matFiles` directory). Similarly, the plots will be saved in four different formats (pdf, svg, png and fig) in the specified output folder (by default, in the `./outputs/plots` directory)

## Usage 

To run this code:

- Setup the case by modifying the files **findBladeWake_stats.m** and **SetupControlPoints.m** for the wake-informed lifting line solution. Also setup the wake vortex system file, by using **SetupHelicalSystem_J60_noContraction.m** or **SetupHelicalSystem_J110_noContraction.m** as an example.

- Modify the **LILSolution.m** file, if load-informed LL solution is desired and if the blade loading is present from external source.

- Adjust the **Mainfile.m** accordingly and evaluate your case by running **Mainfile.m** file in Matlab.

When running the **Mainfile.m** as provided, you should see a new .mat file in `./outputs/matFiles/` containing the output of the code. Additionally, the curvefit, helix and control points plots should be saved in `./outputs/plots/` directory when running with the default inputs.


## Author(s)

This software has been developed by
**Jatinder Goyal** ![ORCID logo](https://info.orcid.org/wp-content/uploads/2019/11/orcid_16x16.png) [0000-0001-7448-8617](https://orcid.org/0000-0001-7448-8617), Technische Universiteit Delft


## License

The function `simps.m` is licensed under **license_simps** (see [license_simps](LICENSES/license_simps.txt) file) with copyright &copy; 2009, Damien Garcia.   

The contents in the `data/` directory are licensed under a **CC-BY 4.0** (see [CC-BY-4.0](LICENSES/CC-BY-4.0.txt) file). The source code and any other file in this repository are licensed under an **Apache License v2.0** (see [Apache-License-v2.0](LICENSES/Apache-License-v2.0.txt) file).

Copyright notice:

Technische Universiteit Delft hereby disclaims all copyright interest in the program “Wake-informed Lifting Line Theory: Example Code” (meaning the source code files licensed under **Apache License v2.0** as explained above). It is an exmaple Matlab code for wake-informed lifting line theory, written by the Author(s).    
Henri Werij, Dean of Faculty of Aerospace Engineering, Technische Universiteit Delft.

&copy; 2025, J. Goyal


## References
 
- Goyal, J., Sinnige, T., Avallone, F., and Ferreira, C., “Estimation of Rotor Blade Loading Distribution From Slipstream Velocity Measurements,” AIAA Journal, 2025. [https://doi.org/10.2514/1.J064736](https://doi.org/10.2514/1.J064736)
- Damien Garcia (2023). Simpson's rule for numerical integration [https://www.mathworks.com/matlabcentral/fileexchange/25754-simpson-s-rule-for-numerical-integration](https://www.mathworks.com/matlabcentral/fileexchange/25754-simpson-s-rule-for-numerical-integration), MATLAB Central File Exchange. Retrieved November 6, 2023.  

## Cite this repository

If you use this software, please cite it as below or check out the `CITATION.cff` file.

**How to cite this repository**: Goyal, J., 2025. Wake-informed Lifting Line Theory: Example Code. 4TU.ResearchData. Software. [https://doi.org/10.4121/4bf0f63f-7ac2-48dc-a2b1-e3fb819b34c2](https://doi.org/10.4121/4bf0f63f-7ac2-48dc-a2b1-e3fb819b34c2) 


## Would you like to contribute?

If you have any comments, feedback, or recommendations, feel free to **reach out** by sending an email to jatindergoel777@gmail.com

Thank you, and enjoy!

