%{
Function to calcuate indcution matrix for the input query points

Copyright 2025 Jatinder Goyal

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
%}
%%
function queryPoints = InductionMatrix(ll,B,D,queryPoints)
% Calculate the induction matrix for vortex strengths affecting query points
%
% Inputs:
%   ll           - Structure containing vortex system properties
%   B            - Number of blades
%   D            - Diameter of the rotor
%   queryPoints  - Structure containing coordinates of query points (xR, yR, zR)
%
% Output:
%   queryPoints  - Updated structure with induced velocity coefficients

% vortex core for Biot-Savart calculations
cnvg.cutoff = 0.01;
for i = 1:B
    [ytmp, ztmp]= rotate(ll.d2y_vortex_rR,ll.d2z_vortex_rR,(i-1)*360/B); %rotate input in meters (not normalized)
    [ytmp_bound, ztmp_bound] = rotate(ll.rR*D/2,zeros(1,length(ll.rR)),(i-1)*360/B);
    for ridx = 1:length(ll.d2x_vortex_rR(:,1))
        [d3aU(:,:,i),d3aV(:,:,i),d3aW(:,:,i)] = biotSavart(queryPoints.xR*D/2,queryPoints.yR*D/2,queryPoints.zR*D/2,...
            ll.d2x_vortex_rR(ridx,:),ytmp(ridx,:),ztmp(ridx,:),cnvg.cutoff);
        d3aU_rR(:,:,i,ridx) = d3aU(:,:,i);
        d3aV_rR(:,:,i,ridx) = d3aV(:,:,i);
        d3aW_rR(:,:,i,ridx) = d3aW(:,:,i);
        clear d3aU d3aV d3aW
    end
    clear ytmp ztmp

    [d3aU_bound(:,:,i),d3aV_bound(:,:,i),d3aW_bound(:,:,i)] = biotSavartBound(queryPoints.xR*D/2,queryPoints.yR*D/2,queryPoints.zR*D/2,...
        zeros(1,length(ll.rR))',ytmp_bound',ztmp_bound',cnvg.cutoff);

    clear ytmp_bound ztmp_bound
end

% Sum the contributions from all vortex segments and blades
queryPoints.UindCoeff = squeeze(sum(sum(d3aU_rR,3),2));
queryPoints.VindCoeff = squeeze(sum(sum(d3aV_rR,3),2));
queryPoints.WindCoeff = squeeze(sum(sum(d3aW_rR,3),2));

queryPoints.UindCoeff_bound = squeeze(sum(d3aU_bound,3));
queryPoints.VindCoeff_bound = squeeze(sum(d3aV_bound,3));
queryPoints.WindCoeff_bound = squeeze(sum(d3aW_bound,3));
