%{
This script calcuates the Load-informed LL solution for the given propeller
blade loading and computes the corresponding lift and drag coefficients. 
It iterates until the solution converges based on the gamma-bound differences.

Copyright 2025 Jatinder Goyal

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
%}
%%
%% SRANS
% Load blade loading data
srans.bladeLoading = importdata([dataPath ,sprintf('ThrustTorque_perBlade_J%.02fV30b15.csv',J)]).data;

srans.rR = srans.bladeLoading(:,1);
srans.dr = gradient(srans.rR*D/2);

% Extract thrust and torque distribution data
srans.dTNdr_P = srans.bladeLoading(:,4); % pressure component
srans.dTNdr_ST = srans.bladeLoading(:,6); % shear component
srans.dTNdr = srans.bladeLoading(:,2); % total
srans.dQNdr_P = srans.bladeLoading(:,5); % pressure component
srans.dQNdr_ST = srans.bladeLoading(:,7); % shear component
srans.dQNdr = srans.bladeLoading(:,3); % total


%% SRANS Load-informed LL

% Interpolate chord and twist at given radial locations
LIL.chord = interp1(prop.geom.rR, prop.geom.data(4,:), WIL.rR,'spline','extrap')./(1000)*prop.scaleFactor; %chord in meters
LIL.twist = interp1(prop.geom.rR, prop.geom.data(3,:), WIL.rR,'spline','extrap')- prop.twist0; %twist in degrees without pitch

% Interpolate thrust and torque data
LIL.dTNdr = interp1(srans.rR,srans.dTNdr,WIL.rR,'spline'); %[N/m] thrust per unit span per blade
LIL.dQNdr = interp1(srans.rR,srans.dQNdr,WIL.rR,'spline'); %[N/m] torque per unit span per blade
LIL.dT = LIL.dTNdr.*B.*WIL.dr;
LIL.dQ = LIL.dQNdr.*B.*WIL.dr;

% Initialize gamma and convergence criteria
LIL.gammaBound = WIL.GammaBound_mean;

LIL.iiter = 1;
LIL.GammaTol = 1e-15; % Tolerance for convergence
LIL.GammaDiff = 1; % Initial gamma difference
LIL.convgF = 0.5; % Relaxation factor for convergence
LIL.maxIter = 1000; % Maximum number of iterations

while max(abs(LIL.GammaDiff)) > LIL.GammaTol
    % Compute circulation in the wake using current bound circulation
    LIL.gammaWake = -[LIL.gammaBound(1); diff(LIL.gammaBound); -LIL.gammaBound(end)];

    % Compute induced velocities using current gamma bounds
    LIL.UInd = WIL.blade.UindCoeff*LIL.gammaWake + WIL.blade.UindCoeff_bound*LIL.gammaBound;
    LIL.VInd = WIL.blade.VindCoeff*LIL.gammaWake + WIL.blade.VindCoeff_bound*LIL.gammaBound;
    LIL.WInd = WIL.blade.WindCoeff*LIL.gammaWake + WIL.blade.WindCoeff_bound*LIL.gammaBound;

    %%
    LIL.Va = V+LIL.UInd;
    LIL.Vt = 2*pi*n*WIL.blade.y + LIL.WInd;
    LIL.Vp = sqrt(LIL.Va.^2 + LIL.Vt.^2);
    LIL.phi = atan2d(LIL.Va, LIL.Vt);
    LIL.alpha = -LIL.phi + LIL.twist + pitch;

    dTf = LIL.dT./(0.5*B*rho.*LIL.Vp.^2.*LIL.chord.*WIL.dr);
    dQf = LIL.dQ./(0.5*B*rho.*LIL.Vp.^2.*LIL.chord.*WIL.rR*D/2.*WIL.dr);

    LIL.Cl = +dTf.*cosd(LIL.phi) + dQf.*sind(LIL.phi);
    LIL.Cd = -dTf.*sind(LIL.phi) + dQf.*cosd(LIL.phi);

    % Update gamma based on calcuated Cl
    gammaBoundNew = (0.5*LIL.Vp.*LIL.Cl.*LIL.chord);

    % Compute difference between old and new gamma bounds
    LIL.GammaDiff = gammaBoundNew - LIL.gammaBound;

    % Update gamma using relaxation factor
    LIL.gammaBound = (1- LIL.convgF)*LIL.gammaBound + LIL.convgF*gammaBoundNew;

    % Increment iteration count
    LIL.iiter = LIL.iiter+1;
    % Break if maximum number of iterations is reached
    if LIL.iiter >= LIL.maxIter
        warning('Maximum number of iterations reached before convergence.');
        break;
    end
    fprintf('Iteration %g, error: %g \n',LIL.iiter,max(abs(LIL.GammaDiff)))

    clear  dTf dQf
end
%% Compute Output Thrust and Torque
LIL.dT_out = 0.5*B*rho*LIL.Vp.^2.*(LIL.Cl.*cosd(LIL.phi) - LIL.Cd.*sind(LIL.phi)).*LIL.chord.*WIL.dr;
LIL.dQ_out = 0.5*B*rho*LIL.Vp.^2.*(LIL.Cl.*sind(LIL.phi) + LIL.Cd.*cosd(LIL.phi)).*LIL.chord.*WIL.rR*D/2.*WIL.dr;

LIL.dTNdr_out =  LIL.dT_out./(B.*WIL.dr); %[N/m] thrust per unit span per blade
LIL.dQNdr_out =  LIL.dQ_out./(B.*WIL.dr); %[Nm/m] torque per unit span per blade
%% Plot Results
figure;
plot(LIL.gammaBound/(V*D),WIL.rR)
plot(WIL.GammaBound_mean/(V*D),WIL.rR)
xlabel('Non-dimensional bound circulation $\Gamma^*$')
ylabel('Radial coordinate $r/R$')
ylim([0 1.2])
xtickformat('%.02f')
ytickformat('%.01f')
yticks(0:0.2:1.2)