%{
Main script for running the wake-informed lifting line code.

Copyright 2025 Jatinder Goyal

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
%}
%%
% Clear variables and close figures
clearvars
clc
close all

matlabPlotSettings; % to set the plotting properties desired for the code
% Input parameters
dataSet = 'srans'; % Dataset type for reading and setting up helical wake system in findBladeWake_stats.m file
dataPath = '../data/SRANS/'; %path to the slipstream data
J = 0.60; % Advance ratio
D = 0.4064; % Diameter in meters
V = 30; % Velocity in m/s
B = 3; % Number of blades
pitch = 15; % Pitch angle in degrees
propGeom = 1; %switch to indicate propeller geoemtry avaialble or not

rR_root = 0.24; % Root vortex locations
if J == 0.60
    xR_minCP = 0.2; % minimum axial location for control points
elseif J ==1.10
    xR_minCP = 0.3; % minimum axial location for control points
end

% Define tolerance for the proximity of the control points to the wake
% structures
wake_tol = 0.01; %x/R

drR = 0.04;
% Density based on advance ratio
if J == 0.60
    rho = 1.2306086; %kg/m^3
elseif J == 1.10
    rho = 1.2301707; %kg/m^3
end


rR_input = 0.30:drR:(1-drR);% selected minimum r/R location to avoid having
% lifting line element in the radial station where there is no PIV data,
% which otherwise lead to increased uncertainity at that radial station


% Number of passes for iterations
npass = 10;

% Flags for saving and plotting
matSave = 1;
matSaveDir = '../outputs/matFiles/';
curvefitplot = 1;
curveFitPlotSave = 1;
helicalSystemPlot = 1;
helicalSystemPlotSave = 1;
controlPointPlot = 1;
controlPointFigSave = 1;
savePlotDir = '../outputs/plots/'; %% directory for saving plots
%% Calculate the rotational speed
n = V / (J * D);
%% Find the location of blade wake and tip vortices
findBladeWake_stats;
if curvefitplot
    curveFittingPlot;
end

%% calcuate in-plane vorticity
[inData.CURLZ, ~] = curl(inData.xR * D / 2, inData.rR * D / 2, inData.Va_Vinf * V, inData.Vr_Vinf * V);
inData.CURLZ_norm = inData.CURLZ * D / V;

%% Setup helical system based on advance ratio
if J == 0.60
    SetupHelicalSystem_J60_noContraction;
elseif J == 1.10
    SetupHelicalSystem_J110_noContraction;
end
if helicalSystemPlot
    helixWakePlot;
end
%% Perform multiple passes to obtain statistically converged solution for wake-informed LL
for np = 1:npass
    SetupControlPoints;
    SolveWakeInformedLL;
    WIL.GammaBound(:, np) = GammaBoundSol;
    WIL.controlPoint{:, np} = controlPoint;
    WIL.error(:, np) = sqrt(sum(CPerror(:).^2));
    fprintf('Progress = %.02f%%\n', np / npass * 100)
end

% Calculate mean and standard deviation of bound circulation
WIL.rR = (ll.rR(2:end) + ll.rR(1:end-1))' / 2;
WIL.dr = diff(ll.rR)';
WIL.GammaBound_std = std(WIL.GammaBound, [], 2);
WIL.GammaBound_mean = mean(WIL.GammaBound, 2);

%% Calculate parameters at the blades
WIL.blade.xR = zeros(length(WIL.rR), 1);
WIL.blade.yR = WIL.rR;
WIL.blade.zR = zeros(length(WIL.rR), 1);
WIL.blade = InductionMatrix(ll, B, D, WIL.blade);
WIL.blade.y = WIL.blade.yR * (D / 2); %[m] dimensionalized

% Compute induced velocities
for np = 1:npass
    GammaBoundSol = WIL.GammaBound(:, np);
    [~, GammaWakeSol, ~, ~, ~] = error_velocity(GammaBoundSol, controlPoint, SlipstreamData);
    WIL.blade.UInd(:, np) = WIL.blade.UindCoeff * GammaWakeSol + WIL.blade.UindCoeff_bound * GammaBoundSol;
    WIL.blade.VInd(:, np) = WIL.blade.VindCoeff * GammaWakeSol + WIL.blade.VindCoeff_bound * GammaBoundSol;
    WIL.blade.WInd(:, np) = WIL.blade.WindCoeff * GammaWakeSol + WIL.blade.WindCoeff_bound * GammaBoundSol;
end

% Compute resultant velocities and forces
WIL.blade.Va = V + WIL.blade.UInd; % [m/s] Axial veloccity
WIL.blade.Vt = 2 * pi * n * WIL.blade.y + WIL.blade.WInd; %[m/s] Tangential veloccity
WIL.blade.Vp = sqrt(WIL.blade.Va.^2 + WIL.blade.Vt.^2); %[m/s] Local veloccity at blade section
WIL.blade.phi = atan2d(WIL.blade.Va, WIL.blade.Vt); %[deg] effective inflow angle
WIL.blade.dTNdr = (rho * WIL.blade.Vp .* WIL.GammaBound) .* cosd(WIL.blade.phi); %[N/m] thrust per unit span per blade
WIL.blade.Ct = WIL.blade.dTNdr ./ (rho .* n.^2 .* D^3); % sectional thrust coefficient
WIL.blade.dTc = WIL.blade.dTNdr ./ (rho .* V.^2 .* D); % sectional thrust coefficient based on dynamic pressure
WIL.blade.L = rho * WIL.blade.Vp .* WIL.GammaBound; % [N/m] sectional Lift force

% Compute thrust and other parameters
WIL.dT = B * (rho * WIL.blade.Vp .* WIL.GammaBound) .* cosd(WIL.blade.phi) .* WIL.dr; %[N] sectional thrust force
WIL.T = B * simps(WIL.rR * D / 2, WIL.blade.dTNdr); %[N] total thrust force by propeller (only lift component)
WIL.CT = WIL.T ./ (rho .* n.^2 .* D.^4); % thrust coefficient
WIL.TC = WIL.T ./ (rho .* V.^2 .* D.^2); % thrust coefsficient based on dynamic pressure

% Parameters requiring propeller geometry
if propGeom
    propellerGeoemtry;
    WIL.blade.chord = interp1(prop.geom.rR, prop.geom.data(4, :), WIL.rR, 'spline', 'extrap') / (1000) * prop.scaleFactor; % Chord in meters
    WIL.blade.twist = interp1(prop.geom.rR, prop.geom.data(3, :), WIL.rR, 'spline', 'extrap') - prop.twist0; % Twist in degrees without pitch

    WIL.blade.cl = 2 * WIL.GammaBound ./ (WIL.blade.Vp .* WIL.blade.chord); % sectional lift coefficient
    WIL.blade.alpha = -WIL.blade.phi + WIL.blade.twist + pitch; % [deg] local angle of attack
end
%% Calculate geometric parameters
WIL.geom.Va = V;
WIL.geom.Vt = 2 * pi * n * WIL.rR * D / 2;
WIL.geom.Vp = sqrt(WIL.geom.Va^2 + WIL.geom.Vt.^2);
WIL.geom.phi = atan2d(WIL.geom.Va, WIL.geom.Vt);
WIL.geom.L = WIL.blade.L .* cosd(WIL.blade.phi - WIL.geom.phi);
WIL.geom.dTNdr = WIL.geom.L .* cosd(WIL.geom.phi);
WIL.geom.Ct = WIL.geom.dTNdr ./ (rho .* n.^2 .* D^3);
WIL.geom.dTc = WIL.geom.dTNdr ./ (rho .* V.^2 .* D);

WIL.geom.gamma = WIL.geom.L ./ (rho * WIL.geom.Vp);
WIL.geom.cl = 2 * WIL.geom.gamma ./ (WIL.geom.Vp .* WIL.blade.chord);
WIL.geom.alpha = -WIL.geom.phi + WIL.blade.twist + pitch;

%% Calculate load-informed LL solution
if strcmp(dataSet, 'srans')
    LILSolution;
end

%% Calculate SRANS parameters based on geometry
if strcmp(dataSet, 'srans')
    srans.chord = interp1(prop.geom.rR, prop.geom.data(4, :), srans.rR, 'spline', 'extrap') / (1000) * prop.scaleFactor; % Chord in meters
    srans.twist = interp1(prop.geom.rR, prop.geom.data(3, :), srans.rR, 'spline', 'extrap') - prop.twist0; % Twist in degrees without pitch

    srans.Geom.Va = V;
    srans.Geom.Vt = 2 * pi * n * srans.rR * D / 2;
    srans.Geom.Vp = sqrt(srans.Geom.Va^2 + srans.Geom.Vt.^2);
    srans.Geom.phi = atan2d(srans.Geom.Va, srans.Geom.Vt);

    srans.Geom.L = +srans.dTNdr .* cosd(srans.Geom.phi) + srans.dQNdr ./ (srans.rR * D / 2) .* sind(srans.Geom.phi);
    srans.Geom.D = -srans.dTNdr .* sind(srans.Geom.phi) + srans.dQNdr ./ (srans.rR * D / 2) .* cosd(srans.Geom.phi);

    srans.Geom.dTNdr = srans.Geom.L .* cosd(srans.Geom.phi);
    srans.Geom.Ct = srans.Geom.dTNdr ./ (rho .* n.^2 .* D^3);
    srans.Geom.dTc = srans.Geom.dTNdr ./ (rho .* V.^2 .* D);

    srans.Geom.gamma = srans.Geom.L ./ (rho * srans.Geom.Vp);
    srans.Geom.cl = 2 * srans.Geom.gamma ./ (srans.Geom.Vp .* srans.chord);
    srans.Geom.alpha = -srans.Geom.phi + srans.twist + pitch;
end

%% Save results to .mat file if enabled
if matSave
    clear CAV cfdData controlPoint CPerror GammaBound0 gammaBoundNew GammaBoundSol GammaWakeSol highVorticityIndices1
    clear highVorticityIndices2 i idx idx1 idx3 iWake iwa minPointsRequired nanCols nanRows np ridx rRGrid sample_indices SInterp smoothedVorticity
    clear UInd VInd VaVinf VaVinf_normalized vorticityData weight_distribution WInd x_sampled xData x_grid xRGrid y_sampled yData y_grid ax cskip exponent
    clear fg figName2 fontsize hcb idx2 idxBL normVar_normalized normVar cb cnvg CURLZ CURLZ_norm iwake

    save([matSaveDir, sprintf('WIL_LIL_J%g_%s_npass%g.mat', J * 100, dataSet,npass)])
end
