%{
Script for defining the positions of control points for the wake-informed lifting line code.

Copyright 2025 Jatinder Goyal

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
%}
%% Setup Random Control Points

% Identify rows and columns with NaN values
nanRows = all(isnan(inData.CURLZ), 2);
nanCols = all(isnan(inData.CURLZ), 1);

% Filter out NaNs
x_grid = inData.xR(~nanRows, ~nanCols);
y_grid = inData.rR(~nanRows, ~nanCols);
normVar = abs(inData.CURLZ(~nanRows, ~nanCols));

exponent = 0; % Adjust this exponent as needed, to adjut the weighting function

% Normalize 'normVar' to be in the range [0, 1]
normVar_normalized = (normVar - min(normVar(:))) / (max(normVar(:)) - min(normVar(:)));

% Number of points you want to sample
num_points = 1000; % Adjust this as needed


% Weighted distribution of control points
weight_distribution = normVar_normalized(:).^ exponent;

% Sample points based on the weighted distribution
sample_indices = randsample(length(weight_distribution), num_points, true, weight_distribution);

% Extract the sampled x and y coordinates
x_sampled = x_grid(sample_indices);
y_sampled = y_grid(sample_indices);

%% Filter the control points based on multiple criterias

% Define interpolation functions
SInterp.VaVinf = scatteredInterpolant(inData.xR(:),inData.rR(:),inData.Va_Vinf(:),'natural');
SInterp.VrVinf = scatteredInterpolant(inData.xR(:),inData.rR(:),inData.Vr_Vinf(:),'natural');
SInterp.VtVinf = scatteredInterpolant(inData.xR(:),inData.rR(:),inData.Vt_Vinf(:),'natural');
SInterp.CURLZ_norm = scatteredInterpolant(inData.xR(:),inData.rR(:),inData.CURLZ_norm(:),'natural');

% % Define tolerance for the proximity of the control points
% wake_tol = 0.01; %x/R

% Filter out points closer to the lifting line filament than wake_tol
idx1 = inpolygon(x_sampled,y_sampled,[fitX1-wake_tol,flip(fitX1+wake_tol)],[fitY1,flip(fitY1)]); %blade wake 1
x_sampled = x_sampled(~idx1);
y_sampled = y_sampled(~idx1);
if J == 0.60
    idx2 = inpolygon(x_sampled,y_sampled,[fitX2-wake_tol,flip(fitX2+wake_tol)],[fitY2,flip(fitY2)]); %blade wake 2
    x_sampled = x_sampled(~idx2);
    y_sampled = y_sampled(~idx2);
end
% Boundary layer filtering
if ~strcmp(dataSet,'experiment')
    idxBL = find(y_sampled>0.27); %% boundary layer
    x_sampled = x_sampled(idxBL);
    y_sampled = y_sampled(idxBL);
end

% Filter based on blade thickness effect
idx3 = find(x_sampled>=xR_minCP);
x_sampled = x_sampled(idx3);
y_sampled = y_sampled(idx3);

% Define control points
controlPoint.xR = x_sampled;
controlPoint.rR = y_sampled;
controlPoint.zR = zeros(size(controlPoint.rR));

% define induced velocities at control points based on input slipstream data
SlipstreamData.Uindvel = SInterp.VaVinf(controlPoint.xR,controlPoint.rR)*V - V;
SlipstreamData.Vindvel = SInterp.VrVinf(controlPoint.xR,controlPoint.rR).*V;
SlipstreamData.Windvel = SInterp.VtVinf(controlPoint.xR,controlPoint.rR).*V;
SlipstreamData.UInf    = V;
%% figure to show the control points
if controlPointPlot && np==npass
    figure();
    surf(inData.xR,inData.rR,zeros(size(inData.xR)),inData.CURLZ_norm,'EdgeColor','none','FaceColor','interp')
    colormap(parula(11))
    caxis([-5.5 5.5])

    % Plot blade wakes and control points
    plot(fitX1,fitY1,'k')
    plot([fitX1-wake_tol,flip(fitX1+wake_tol)],[fitY1,flip(fitY1)],'--k')
    if J == 0.60
        plot(fitX2,fitY2,'k')
        plot([fitX2-wake_tol,flip(fitX2+wake_tol)],[fitY2,flip(fitY2)],'--k')
    end
    plot(controlPoint.xR,controlPoint.rR,'w+','LineWidth',1,'MarkerSize',4)
    plotGeom_2D;
    axis equal
    xlabel('Axial coordinate $x/R$')
    ylabel('Radial coordinate $r/R$')
    xlim([-inf 1.2])
    ylim([0 1.2])
    xtickformat('%.01f')
    ytickformat('%.01f')
    xticks(0:0.3:1.2)
    yticks(0:0.2:1.2)
    if controlPointFigSave
        figName2 = sprintf('%s_controlPoints_J%.02f',dataSet,J);

        saveas(gcf,[savePlotDir,figName2,'.fig'],'fig')
        saveas(gcf,[savePlotDir,figName2,'.png'],'png')
        saveas(gcf,[savePlotDir,figName2,'.svg'],'svg')
        saveas(gcf,[savePlotDir,figName2,'.pdf'],'pdf')
    end
end
