%{
Calcualte induced velocities for each vortex line using Biot Savart's law

Copyright 2025 Jatinder Goyal

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
%}
%%
function [d2velU,d2velV,d2velW] = biotSavart(xCP,yCP,zCP,d2x_vortex,d2y_vortex,d2z_vortex,cutoff)
% Calculate the induced velocities using the Biot-Savart law for a vortex line
%
% Inputs:
%   xCP, yCP, zCP     - Coordinates of the control points
%   d2x_vortex, d2y_vortex, d2z_vortex - Vortex segment coordinates
%   cutoff - Cutoff distance for the Biot-Savart law
%
% Outputs:
%   d2velU, d2velV, d2velW - Induced velocities in U, V, and W directions

% Number of vortex line segments and elements
iWL = size(d2x_vortex,2); % Number of vortex line segment
iElem = size(d2x_vortex,1); % Number of elements in each vortex line segment

% Initialize arrays for vortex segment data
d1inflPX = zeros(1,iWL*iElem);
d1inflPY = zeros(1,iWL*iElem);
d1inflPZ = zeros(1,iWL*iElem);
d1Circ   = zeros(1,iWL*iElem);


for i = 1:iElem
    d1inflPX((i-1)*iWL+1:i*iWL) = d2x_vortex(i,:);
    d1inflPY((i-1)*iWL+1:i*iWL) = d2y_vortex(i,:);
    d1inflPZ((i-1)*iWL+1:i*iWL) = d2z_vortex(i,:);

    d1Circ((i-1)*iWL+1:i*iWL) = [ones(1,iWL-1),0];
end


d1Circ = d1Circ(1:end-1);

[d2velU,d2velV,d2velW] = vortexLine(xCP,yCP,zCP,d1inflPX(2:end),d1inflPY(2:end),d1inflPZ(2:end),...
    d1inflPX(1:end-1),d1inflPY(1:end-1),d1inflPZ(1:end-1),d1Circ,cutoff);



