%{
Function to calcuate the velocity error between induced velocities calculated 
using wake-informed LL and the input measured velocities

Copyright 2025 Jatinder Goyal

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
%}
%%
function [error,varargout] = error_velocity(gammaBound,controlPoint,PIV)
% Calculate the velocity error between induced velocities and measured velocities
%
% Inputs:
%   gammaBound  - Vector of circulation strengths for the vortex bound elements
%   controlPoint - Structure containing control points and the calcuated induced velocities
%   PIV         - Structure containing measured velocities and reference velocity
%
% Outputs:
%   error       - Vector of errors in velocity components (U, V, W)
%   varargout   - Optional outputs: gammaWake, UInd, VInd, WInd

% Compute the circulation strengths for the wake vortices
gammaWake = -[gammaBound(1); diff(gammaBound); -gammaBound(end)];

% Compute the induced velocities at control points due to the wake vortices
UInd = controlPoint.UindCoeff*gammaWake + controlPoint.UindCoeff_bound*gammaBound;
VInd = controlPoint.VindCoeff*gammaWake + controlPoint.VindCoeff_bound*gammaBound;
WInd = controlPoint.WindCoeff*gammaWake + controlPoint.WindCoeff_bound*gammaBound;

% Calculate the error as the difference between induced velocities and measured velocities, normalized by reference velocity
error = [UInd - PIV.Uindvel, VInd - PIV.Vindvel, WInd - PIV.Windvel]/PIV.UInf;

% Assign optional outputs
varargout{1} = gammaWake; % Wake circulation strength
varargout{2} = UInd; % Induced U velocity component
varargout{3} = VInd; % Induced V velocity component
varargout{4} = WInd; % Induced W velocity component