%{
Script for finding the location of the blade wake and tip vortex location 
for the wake-informed lifting line code.

Copyright 2025 Jatinder Goyal

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
%}
%%
% Define grid for interpolation
xRGrid = 0.06:0.01:1.15;
rRGrid = 0.25:0.01:1.15;
%% import slipstream data based on the dataset type

% Define the path for SRANS data
srans.path = dataPath;
srans.data = importdata([srans.path,sprintf('phaseLocked_J%.02fV%gb%g.xlsx',J,V,pitch)]);

% Create scatteredInterpolant objects for SRANS data
cfdData.FVa = scatteredInterpolant(srans.data.data(:,2),srans.data.data(:,3),srans.data.data(:,6)/V,'natural','none');
cfdData.FVr = scatteredInterpolant(srans.data.data(:,2),srans.data.data(:,3),srans.data.data(:,7)/V,'natural','none');
cfdData.FVt = scatteredInterpolant(srans.data.data(:,2),srans.data.data(:,3),srans.data.data(:,8)/V,'natural','none');

%% Create mesh grid for interpolation
[inData.xR,inData.rR] = meshgrid(xRGrid,rRGrid);
inData.zR = zeros(size(inData.xR));

% Interpolate data
inData.Va_Vinf = cfdData.FVa(inData.xR,inData.rR);
inData.Vr_Vinf = cfdData.FVr(inData.xR,inData.rR);
inData.Vt_Vinf = cfdData.FVt(inData.xR,inData.rR);

% Calculate in-plane vorticity (not with proper dimensions as the input
% grid as well as velocity values are non-dimensionalized) for
% wake identification
[CURLZ, CAV] = curl(inData.xR * D / 2, inData.rR * D / 2, inData.Va_Vinf * V, inData.Vr_Vinf * V);
CURLZ_norm = CURLZ*D/V;

%%
% Load your grid data (x, y, and vorticity values) into arrays

% Filter or preprocess your data to remove noise if necessary
% You can use techniques like smoothing or outlier removal here

% Convert data to vectors for processing
xData = inData.xR(:);
yData = inData.rR(:);
vorticityData = CURLZ(:);

% Remove NaN values
idx = isnan(vorticityData);
xData = xData(~idx);
yData = yData(~idx);
vorticityData = vorticityData(~idx);

% Apply smoothing to the vorticity data
smoothedVorticity = smoothdata(vorticityData, 'gaussian', 10); % Adjust the window size as needed

% % Define threshold functions based on the advance ratio (needs adaptation
% as per the the input data/operating condition)
% You can define your own function based on your requirements
if J == 0.60
    wt_critical = 3.5; %threshold vorticity
    thresholdFunction = @(y) wt_critical*V/D; % applied a constant absolute value threshhold based on the values of CURLZ for srans

    % Find the high vorticity regions based on the threshold function,
    % and additional contraints defined based on the visual of CURLZ
    % surface plot
    highVorticityIndices1 = find(abs(smoothedVorticity) > thresholdFunction(yData) & xData <0.6 & yData <= 1.0 & yData >=0.29); % blade wake 1
    highVorticityIndices2 = find(abs(smoothedVorticity) > thresholdFunction(yData) & xData >0.6 & yData <= 1.0 & xData <= 0.999 & yData >=0.29); % blade wake 2

    highVorticityX1 = xData(highVorticityIndices1);
    highVorticityY1 = yData(highVorticityIndices1);
    highVorticityX2 = xData(highVorticityIndices2);
    highVorticityY2 = yData(highVorticityIndices2);

    % In cases where there is no slisptream data avaialble near the
    % nacelle (y/R<=0.30), it might become important to give some fictitious points to
    % obtain a ploynomial fit that provides a physical shape. Here it is
    % assumed that the convection velocity at the root can be assumed to be
    % same as the convection velocity at the tip for the positive thrust
    % conditons. This condition **does not** kick in for the sample srans
    % data.
    if min(highVorticityY1)>0.3
        idx = find(highVorticityY1 <= 1.05 & highVorticityY1 >= 0.95); % Identify indices for y close to 1
        highVorticityY1 = [0.25*ones(length(idx),1); highVorticityY1]; %% asign the x-location of the fictitious points at y/R = 0.25 same as at tip
        highVorticityX1 = [highVorticityX1(idx); highVorticityX1];
    end
    if min(highVorticityY2)>0.3
        idx = find(highVorticityY2 <= 1.05 & highVorticityY2 >= 0.95); % Identify indices for y close to 1
        highVorticityY2 = [0.25*ones(length(idx),1); highVorticityY2]; %% asign the x-location of the fictitious points at y/R = 0.25 same as at tip
        highVorticityX2 = [highVorticityX2(idx); highVorticityX2];
    end

    %%This loop is there to determine the appropriate polynimial order for
    %%the blade wakes
    for odr = 1:9
        poly = sprintf('poly%g',odr);
        mdlWake1{odr} = fitlm(highVorticityY1, highVorticityX1,poly);
        mdlWake2{odr}  = fitlm(highVorticityY2, highVorticityX2,poly);

        rmseWake1(odr) = mdlWake1{odr}.RMSE;
        rmseWake2(odr) = mdlWake2{odr}.RMSE;

        pValWake1{odr} = mdlWake1{odr}.Coefficients.pValue;
        pValWake2{odr} = mdlWake2{odr}.Coefficients.pValue;
    end

    %%These are the chosen polynimial orders based on the above for loop,
    % right now this part is manual and could be automated for a better
    % control.
    modelfunWake1= 'poly3'; %chose 3rd order based on p-values
    polyFitWake1 = fit(highVorticityY1,highVorticityX1,modelfunWake1);

    modelfunWake2= 'poly3'; %chose 3rd order based on p-values
    polyFitWake2 = fit(highVorticityY2,highVorticityX2,modelfunWake2);


    % Generate points on the fitted curve for visualization
    %blade wake 1
    fitY1 = 0.23:0.01:1.0;
    fitX1 = polyFitWake1(fitY1)';
    %blade wake 2
    fitY2 = 0.23:0.01:1.0;
    fitX2 = polyFitWake2(fitY2)';
    %%
elseif J == 1.10
    % Define threshold functions for positive and negative vorticity as per
    % your data

    % Find the high vorticity regions based on the threshold function,
    % and additional contraints defined based on the visual of CURLZ
    % surface plot. Notice that yData>0.29  & xData<0.90 is to avoid
    % boundary layer regions in SRANS data.

    wt_critical_pos = 0.6; %threshold vorticity
    wt_critical_neg = -1.5; %threshold vorticity

    thresholdFunction_pos = @(y) wt_critical_pos*V/D;
    thresholdFunction_neg = @(y) (wt_critical_neg*V/D);

    highVorticityIndices1 = find((smoothedVorticity > thresholdFunction_pos(yData) | smoothedVorticity < thresholdFunction_neg(yData)) & xData >0.3 & yData < 1.1 & yData >  0.29 & xData <=0.90);

    highVorticityX1 = xData(highVorticityIndices1); %% x/R locations with hgh vorticity
    highVorticityY1 = yData(highVorticityIndices1); %% y/R locations with hgh vorticity

    % In cases where there is no slisptream data avaialble near the
    % nacelle (y/R<=0.30), it might become important to give some fictitious points to
    % obtain a ploynomial fit that provides a physical shape. Here it is
    % assumed that the convection velocity at the root should be the lowest
    % for the negative thrust conditon.
    idx = find(highVorticityX1<=min(highVorticityX1)+0.05); %% find the points with lowest axial positions with some tolerance
    highVorticityY1 = [0.25*ones(length(idx),1); 0.3*ones(length(idx),1); highVorticityY1];
    highVorticityX1 = [min(highVorticityX1)*ones(length(idx),1); highVorticityX1(idx); highVorticityX1];

    for odr = 1:9
        poly = sprintf('poly%g',odr);
        mdlWake1{odr} = fitlm(highVorticityY1,highVorticityX1,poly);
        rmseWake1(odr) = mdlWake1{odr}.RMSE;
        pValWake1{odr} = mdlWake1{odr}.Coefficients.pValue;

    end

    modelfunWake1= 'poly6';%fittype('d*x^4 + c*x^3 + b*x^2 + a*x + e'); %chose 6th order based on p-values
    polyFitWake1 = fit(highVorticityY1,highVorticityX1,modelfunWake1);

    % Generate points on the fitted curve for visualization
    fitY1 = 0.23:0.01:1.0;
    fitX1 = polyFitWake1(fitY1)';
end


